@@ -93,19 +93,37 @@ EstPara <- function (ref.otu.tab) {
 #' @export
 
 SimulateMSeq <- function (
-		ref.otu.tab, nSam = 100, nOTU = 500,
-		diff.otu.pct = 0.1, diff.otu.direct = c("balanced", "unbalanced"), diff.otu.mode = c("abundant", "rare", "mix"),
-		covariate.type = c("binary", "continuous"), grp.ratio = 1, covariate.eff.mean = 1, covariate.eff.sd = 0,
-		confounder.type = c("none", "binary", "continuous", "both"), conf.cov.cor = 0.6, 
-		conf.diff.otu.pct = 0, conf.nondiff.otu.pct = 0.1, confounder.eff.mean = 0, confounder.eff.sd = 0,
-		error.sd = 0, depth.mu = 10000, depth.theta = 5,  depth.conf.factor = 0
+		ref.otu.tab, 
+		model.paras = NULL,
+		nSam = 100, 
+		nOTU = 500,
+		diff.otu.pct = 0.1, 
+		diff.otu.direct = c("balanced", "unbalanced"), 
+		diff.otu.mode = c("abundant", "rare", "mix"),
+		covariate.type = c("binary", "continuous"), 
+		grp.ratio = 1, 
+		covariate.eff.mean = 1, 
+		covariate.eff.sd = 0,
+		confounder.type = c("none", "binary", "continuous", "both"), 
+		conf.cov.cor = 0.6, 
+		conf.diff.otu.pct = 0, 
+		conf.nondiff.otu.pct = 0.1, 
+		confounder.eff.mean = 0, 
+		confounder.eff.sd = 0,
+		error.sd = 0, 
+		depth.mu = 10000, 
+		depth.theta = 5,  
+		depth.conf.factor = 0
 ){
 	diff.otu.direct <- match.arg(diff.otu.direct)
 	diff.otu.mode <- match.arg(diff.otu.mode)
 	covariate.type <- match.arg(covariate.type)
 	confounder.type <- match.arg(confounder.type)
 	ref.otu.tab0 <- ref.otu.tab
-	model.paras <- EstPara(ref.otu.tab = ref.otu.tab) # full ref.otu.tab + estimated parameters
+	
+	if (is.null(model.paras)) {
+	  model.paras <- EstPara(ref.otu.tab = ref.otu.tab) # full ref.otu.tab + estimated parameters
+	}
 	
 	sample.names <- colnames(model.paras$ref.otu.tab)
 	ref.otu.tab <- model.paras$ref.otu.tab[(1 : (nOTU)),] # otu * sample
@@ -272,6 +290,7 @@ SimulateMSeq <- function (
 SimulateMSeqC <- function (
 		# Input data
 		ref.otu.tab, # otu * sample
+		model.paras = NULL,
 		nSubject = 40, nOTU = 50, nTime = 2,
 		# Random error
 		error.sd = 1, 
@@ -295,8 +314,11 @@ SimulateMSeqC <- function (
 		if (Z.diff.otu.pct > T.diff.otu.pct)  stop("Z.diff.otu.pct should not be larger than T.diff.otu.pct!\n")
 	}
 	
-	## Estimate parameters
-	model.paras <- EstPara(ref.otu.tab)
+	## Estimate parameters 
+	if (is.null(model.paras)) {
+	  model.paras <- EstPara(ref.otu.tab)
+	}
+	
 	## select top nOTU
 	sample.names <- colnames(model.paras$ref.otu.tab)
 	idx.sample <- sample(sample.names, nSubject, replace = T)
