@@ -264,7 +264,7 @@ ZicoSeq <- function (
 	if (feature.dat.type == 'count' & is.post.sample == FALSE){
 		if (min(coef(summary(lm(sqrt(colSums(feature.dat)) ~  meta.dat[, grp.name])))[-1, 'Pr(>|t|)']) < 0.05) {
 			warning('The sequencing depth is correlated with the variable of interest!
-							Rarefaction is needed to control for false positves!\n')
+							Rarefaction is recommended to control for false positves!\n')
 			
 		}
 	}
@@ -358,6 +358,7 @@ ZicoSeq <- function (
 					}				
 				})
 		feature.dat <- t(feature.dat)
+		
 		sds <- rowSds(feature.dat)
 		if (sum(sds == 0) != 0) {
 			stop(paste('After winsorization, feature ',  paste(which(sds == 0), 
@@ -369,8 +370,11 @@ ZicoSeq <- function (
 	if (feature.dat.type == 'proportion') {
 		# Renormalization
 		feature.dat <- t(t(feature.dat) / colSums(feature.dat))
+		
 	}
 	
+	# Recalculate depth after winsorization (2025-06-25) = 1
+	depth <- colSums(feature.dat)
 	###############################################################################
 	# Generate samples from posterior distribution (stacking it)
 	if (is.post.sample == TRUE) {
@@ -487,7 +491,9 @@ ZicoSeq <- function (
 	if (feature.dat.type %in% c('count', 'proportion')) {
 		if (verbose)  cat('Finding the references ...\n')
 		ref.ind <- find.ref.z1(feature.dat, M0, ref.pct = ref.pct, feature.dat.type = feature.dat.type, p.max = p.max)
-		size.factor <- colSums(feature.dat[ref.ind, ])
+		
+		divisor <- colSums(feature.dat[ref.ind, , drop = FALSE]) / depth
+	
 	} else {
 		ref.ind <- NULL
 	}
@@ -502,8 +508,10 @@ ZicoSeq <- function (
 		if (feature.dat.type == 'other') {
 			divisor <- 1
 		} else {
-			divisor <- size.factor / depth
-			divisor[divisor == 0] <- 0.5 * min(divisor[divisor != 0])
+		  
+			# # Handle zero divisor - half minimum
+			# divisor[divisor == 0] <- 0.5 * min(divisor[divisor != 0])
+		  divisor[divisor  == 0] <- 0.5 * min(divisor [divisor != 0])
 		}
 		
 		# Create the giant Y matrix
@@ -516,7 +524,8 @@ ZicoSeq <- function (
 				feature.dat.p <- feature.dat.p.list[[k]]
 				
 				Y[, (k - 1) * func.no * otu.no + func.no * (0 : (otu.no - 1)) + j] <-
-						func(t(feature.dat.p) / divisor)  # No scaling first
+						func(t(feature.dat.p) / divisor)  
+
 				
 			}
 		}
@@ -611,13 +620,13 @@ ZicoSeq <- function (
 			} else {
 				ind <- which(p.raw <= 0.05)
 			}
-			size.factor <- colSums(feature.dat[setdiff(ref.ind, which.nan.ind[ind]), ])
-			# norm.ind <- cbind(norm.ind, !(1:nrow(feature.dat) %in% setdiff(ref.ind, which.nan.ind[ind])))
+		  
+		  new.ref.ind <- setdiff(ref.ind, which.nan.ind[ind])
+		  divisor <- colSums(feature.dat[new.ref.ind, , drop = FALSE]) / depth
+
 		}
 	}
 	
-	
-	
 	p.adj.fdr <- perm.fdr.adj(F0, Fp)
 	p.raw <- na.pad(p.raw, na.ind)
 	p.adj.fdr <- na.pad(p.adj.fdr, na.ind)
@@ -632,13 +641,22 @@ ZicoSeq <- function (
 		p.adj.fwer <- NULL
 	}
 	
+	# 2025-06-25  better esitmation of the coefficient
 	# Finally, get the direction of change under specific transformations
-    if (length(ref.ind) != 0) {
-    divisor <- colSums(feature.dat[ref.ind, ])
-    divisor[divisor == 0] <- 1
-		temp.dat <- t(t(feature.dat) / divisor)
+#   if (length(ref.ind) != 0) {
+#     divisor <- colSums(feature.dat[ref.ind, ])
+#     divisor[divisor == 0] <- 1
+# 		temp.dat <- t(t(feature.dat) / divisor)
+# 	} else {
+# 		temp.dat <- feature.dat
+# 	}
+	if (feature.dat.type != 'other') {
+	  temp.dat <- Reduce("+", feature.dat.p.list) / length(feature.dat.p.list)
+	  divisor <- colSums(temp.dat[ref.ind, ])
+    divisor[divisor == 0] <- 0.5 * min(divisor[divisor!= 0])
+	  temp.dat <- t(t(temp.dat) / divisor)
 	} else {
-		temp.dat <- feature.dat
+	  temp.dat <- feature.dat.p.list[[1]]
 	}
 	
 	coef.list <- NULL
@@ -692,29 +710,29 @@ ZicoSeq.plot <- function(ZicoSeq.obj, pvalue.type = c('p.adj.fdr','p.raw','p.adj
 	
 	grp.name <-  ZicoSeq.obj$grp.name
 	meta.dat <- ZicoSeq.obj$meta.dat
+	feature.dat <- ZicoSeq.obj$feature.dat
+	feature.dat.type <- ZicoSeq.obj$call$feature.dat.type
 	
-	# cat('Significant(',pvalue.type,'<=',cutoff,') association strength between taxa and ',grp.name, ' is visualized!\n' )
-	
-	if(ZicoSeq.obj$call$feature.dat.type == 'proportion'){
-		abundance <- ZicoSeq.obj$feature.dat
-		prevalence <- apply(ZicoSeq.obj$feature.dat, 1, function(x) mean(x > 0))
+	if (is.null(feature.dat)) {
+	  stop("No 'feature.dat' found. Please set 'return.feature.dat = TRUE' when running ZicoSeq!\n")
 	}
 	
-	if(ZicoSeq.obj$call$feature.dat.type == 'count'){
-		abundance <- t(t(ZicoSeq.obj$feature.dat)/colSums(ZicoSeq.obj$feature.dat))
-		prevalence <- apply(ZicoSeq.obj$feature.dat, 1, function(x) mean(x > 0))
+	
+	if(feature.dat.type %in% c('count', 'proportion')){
+	  feature.dat <-  t(t(feature.dat) / colSums(feature.dat))
+	  prevalence <- rowMeans(feature.dat != 0)
+	  abundance <- rowMeans(feature.dat)
 	}
 	
-	if(ZicoSeq.obj$call$feature.dat.type == 'other'){
-		abundance <- ZicoSeq.obj$feature.dat
-		prevalence <- apply(abundance, 1, function(x) sd(x))
+	
+	if(feature.dat.type == 'other'){
+		abundance <- rowMeans(feature.dat)
+		prevalence <- rowSds(feature.dat)
 	}
 	
-	# relative abundance
-	abundance <- apply(abundance, 1, function(x) mean(x))
-	
+
 	if(!(length(unique(meta.dat[,grp.name])) > 2 & !is.numeric(meta.dat[,grp.name]))) {
-		coefs <- sapply(ZicoSeq.obj$coef.list, function(x) x[grep(grp.name,rownames(x)),])
+		coefs <- sapply(ZicoSeq.obj$coef.list, function(x) x[grep(grp.name, rownames(x)),])
 		colnames(coefs) <- paste0('coef_Func',1:length(ZicoSeq.obj$coef.list))
 	}  
 	
@@ -725,20 +743,17 @@ ZicoSeq.plot <- function(ZicoSeq.obj, pvalue.type = c('p.adj.fdr','p.raw','p.adj
 	}
 	
 	# Break the ties
-	ZicoSeq.obj$R2 <- ZicoSeq.obj$R2 + runif(length(ZicoSeq.obj$R2), 0, 1e-10)
+	ZicoSeq.obj$R2 <- ZicoSeq.obj$R2 + runif(length(ZicoSeq.obj$R2), 0, 1e-5)
 	
 	R2 <- rowMaxs(ZicoSeq.obj$R2)
 
-	## pvalue = 0??? -log10(p)
 	if(length(unique(meta.dat[,grp.name])) > 2 & !is.numeric(meta.dat[,grp.name])){
 		plot.data <- data.frame(pvals = ZicoSeq.obj[[pvalue.type]], 
 				prevalence = prevalence, abundance = abundance, 
 				R2 = R2, 
 				taxa = rownames(ZicoSeq.obj$R2))
 		plot.data[plot.data$pvals > cutoff, 'taxa'] <- ''
-		
-		
-	}else{
+	} else {
 
 		signs <- t(sign(coefs))[t(ZicoSeq.obj$R2 == R2)]
 		signs[signs == 0] <- 1
@@ -749,8 +764,6 @@ ZicoSeq.plot <- function(ZicoSeq.obj, pvalue.type = c('p.adj.fdr','p.raw','p.adj
 				taxa = rownames(ZicoSeq.obj$R2))
 		
 		plot.data[plot.data$pvals > cutoff, 'taxa'] <- ''
-		
-		
 	}
 	
 	if(is.numeric(meta.dat[,grp.name])) {
@@ -775,8 +788,8 @@ ZicoSeq.plot <- function(ZicoSeq.obj, pvalue.type = c('p.adj.fdr','p.raw','p.adj
 			scale_y_continuous(limits = c(0, max(-log10(plot.data$pvals)) * 1.3)) +
 			ggrepel::geom_text_repel(aes(label = taxa), max.overlaps = Inf, color = 'black') +
 			labs(x = bquote(R^2), y = paste0('-log10(',pvalue.type,')'),
-					color = ifelse(ZicoSeq.obj$call$feature.dat.type == 'other','Standard deviation','Prevalence'),
-					size = ifelse(ZicoSeq.obj$call$feature.dat.type == 'other','Mean value','Mean abundance')) +
+					color = ifelse(feature.dat.type == 'other','Standard deviation','Prevalence'),
+					size = ifelse(feature.dat.type == 'other','Mean value','Mean abundance')) +
 			theme_bw() +
 			theme(axis.text = element_text(color = 'black', size = text.size),
 					axis.title = element_text(color = 'black', size = text.size),
@@ -784,10 +797,6 @@ ZicoSeq.plot <- function(ZicoSeq.obj, pvalue.type = c('p.adj.fdr','p.raw','p.adj
 					legend.title = element_text(color = 'black', size = text.size)) +
 			ggtitle(title.name)
 	
-	
-	
-#	plots <- ggarrange(plotlist = plot.list, common.legend = T)
-	
 	if(!is.null(out.dir)) {
 		print(plot.obj)
 		ggsave(paste0(out.dir, file.name), width = width, height = height)
